local util = require "./lua/util"

local string = string

local setmetatable = setmetatable
local DoCall = DoCall

local MF_SetFrameSize = MF_SetFrameSize
local MF_SetChipsets = MF_SetChipsets
local MF_SetFrameColor = MF_SetFrameColor

module (...)

-- ----------------------------------------------------------------------------

function FrameClass ()
	return Frame:new()
end

Frame = {
	filep = { mime = "FRAM", ver = 3, hsz = 372 },
	frameWidth = 30,
	frameHeight = 30,
	tileWidth = 16,
	tileHeight = 16,
	currentZone = 1,
	collisionChip = "",
	backgroundChip = "",
	foregroundChip = "",
	dataDir = "data/",
	levelName = "",
	author = "",
	key = "",
	background_color = 16777215,
}

function Frame:new ()
	obj = {}
	setmetatable (obj, self)
	self.__index = self
	return obj
end

function Frame:GetProperties ()
	return self.frameWidth, self.frameHeight, self.collisionChip, self.backgroundChip, self.foregroundChip, self.dataDir, self.tileWidth, self.tileHeight, self.background_color
end
	
function Frame:import (data, ver)
	self.frameWidth = util.unu16(string.sub(data, 1, 2))
	self.frameHeight = util.unu16(string.sub(data, 3, 4))
	self.collisionChip = util.unfixstr(string.sub(data, 5, 68))
	self.backgroundChip = util.unfixstr(string.sub(data, 69, 132))
	self.foregroundChip = util.unfixstr(string.sub(data, 133, 196))
	
	if (ver >= 1) then
		self.tileWidth = util.unu16(string.sub(data, 197, 198))
		self.tileHeight = util.unu16(string.sub(data, 199, 200))
	end
	
	if (ver >= 2) then
		self.levelName = util.unfixstr(string.sub(data, 201, 264))
		self.author = util.unfixstr(string.sub(data, 265, 296))
		self.key = util.unfixstr(string.sub(data, 297, 360))
	end
	
	if (ver >= 3) then
		self.background_color = util.unu32(string.sub(data, 361, 364))
	end

	MF_SetFrameSize (self.frameWidth, self.frameHeight, self.tileWidth, self.tileHeight)
	MF_SetChipsets (self.dataDir .. self.collisionChip, self.dataDir .. self.backgroundChip, self.dataDir .. self.foregroundChip)
	MF_SetFrameColor (self.background_color)
end
	
function Frame:export ()
	local data = self.filep.mime .. util.u16(self.filep.ver) .. util.u16(self.filep.hsz)
	data = data .. util.u16(self.frameWidth) .. util.u16(self.frameHeight)
	
	data = data .. util.fixstr( self.collisionChip, 64 )
	data = data .. util.fixstr( self.backgroundChip, 64 )
	data = data .. util.fixstr( self.foregroundChip, 64 )
	
	data = data .. util.u16(self.tileWidth) .. util.u16(self.tileHeight)
	
	data = data .. util.fixstr (self.levelName, 64)
	data = data .. util.fixstr (self.author, 32)
	data = data .. util.fixstr (self.key, 64)
	
	data = data .. util.u32 (self.background_color)
	
	return data
end
	
function Frame:reset ()
	self.currentZone = 1
end