local setmetatable = setmetatable
local error = error
local tostring = tostring

module (...)

-- ----------------------------------------------------------------------------

function newList ()
	return List:new()
end

List = { }

function List:new ()
	obj = { first = 0, last = -1 }
	setmetatable (obj, self)
	self.__index = self
	return obj
end

function List:unshift (value)
	local first = self.first - 1
	self.first = first
	self[first] = value
end

function List:push (value)
	local last = self.last + 1
	self.last = last
	self[last] = value
end

function List:shift ()
	local first = self.first
	if (first > self.last) then
		error ("List.shift called on empty list")
	end
	
	local value = self[first]
	self[first] = nil
	self.first = first + 1
	return value
end

function List:pop ()
	local last = self.last
	if (self.first > last) then
		error ("List.pop called on empty list")
	end
	
	local value = self[last]
	self[last] = nil
	self.last = last - 1
	return value
end

function List:size ()
	return self.last - self.first + 1
end

function List:empty ()
	if (self:size() > 0) then
		return false
	end
	
	return true
end

-- ----------------------------------------------------------------------------

function newRollingList (size)
	return RollingList:new (size)
end

RollingList = { }

function RollingList:new (size)
	obj = { max_size = size, list = List:new() }
	setmetatable (obj, self)
	self.__index = self
	return obj
end

function RollingList:setSize (sz)
	self.max_size = sz
	while (self.list:size() > sz) do
		self.list:shift()
	end
end

function RollingList:unshift (value)
	if (self.list:size() >= self.max_size) then
		self.list:pop()
	end
	
	self.list:unshift (value)
end

function RollingList:push (value)
	if (self.list:size() >= self.max_size) then
		self.list:shift()
	end
	
	self.list:push (value)
end

function RollingList:shift ()
	return self.list:shift()
end

function RollingList:pop ()
	return self.list:pop()
end

function RollingList:size ()
	return self.list:size()
end

function RollingList:empty ()
	if (self:size() <= 0) then
		return true
	end
	return false
end

function RollingList:clear ()
	self.list = List:new()
end