local math = math
local string = string
local os = os

local error = error

module (...)

-- --------------------------------------------------------
local shrt_4 = {
	[0] = 0, [16] = 1, [32] = 2, [48] = 3, 
	[64] = 4, [80] = 5, [96] = 6, [112] = 7,
	[128] = 8, [144] = 9, [160] = 10, [176] = 11,
	[192] = 12, [208] = 13, [224] = 14, [240] = 15,
}

local xort = {
	[0] = { [0] = 0, [1] = 1, [2] = 2, [3] = 3, [4] = 4, [5] = 5, [6] = 6, [7] = 7, [8] = 8, [9] = 9, [10] = 10, [11] = 11, [12] = 12, [13] = 13, [14] = 14, [15] = 15, },
	[1] = { [0] = 1, [1] = 0, [2] = 3, [3] = 2, [4] = 5, [5] = 4, [6] = 7, [7] = 6, [8] = 9, [9] = 8, [10] = 11, [11] = 10, [12] = 13, [13] = 12, [14] = 15, [15] = 14, },
	[2] = { [0] = 2, [1] = 3, [2] = 0, [3] = 1, [4] = 6, [5] = 7, [6] = 4, [7] = 5, [8] = 10, [9] = 11, [10] = 8, [11] = 9, [12] = 14, [13] = 15, [14] = 12, [15] = 13, },
	[3] = { [0] = 3, [1] = 2, [2] = 1, [3] = 0, [4] = 7, [5] = 6, [6] = 5, [7] = 4, [8] = 11, [9] = 10, [10] = 9, [11] = 8, [12] = 15, [13] = 14, [14] = 13, [15] = 12, },
	[4] = { [0] = 4, [1] = 5, [2] = 6, [3] = 7, [4] = 0, [5] = 1, [6] = 2, [7] = 3, [8] = 12, [9] = 13, [10] = 14, [11] = 15, [12] = 8, [13] = 9, [14] = 10, [15] = 11, },
	[5] = { [0] = 5, [1] = 4, [2] = 7, [3] = 6, [4] = 1, [5] = 0, [6] = 3, [7] = 2, [8] = 13, [9] = 12, [10] = 15, [11] = 14, [12] = 9, [13] = 8, [14] = 11, [15] = 10, },
	[6] = { [0] = 6, [1] = 7, [2] = 4, [3] = 5, [4] = 2, [5] = 3, [6] = 0, [7] = 1, [8] = 14, [9] = 15, [10] = 12, [11] = 13, [12] = 10, [13] = 11, [14] = 8, [15] = 9, },
	[7] = { [0] = 7, [1] = 6, [2] = 5, [3] = 4, [4] = 3, [5] = 2, [6] = 1, [7] = 0, [8] = 15, [9] = 14, [10] = 13, [11] = 12, [12] = 11, [13] = 10, [14] = 9, [15] = 8, },
	[8] = { [0] = 8, [1] = 9, [2] = 10, [3] = 11, [4] = 12, [5] = 13, [6] = 14, [7] = 15, [8] = 0, [9] = 1, [10] = 2, [11] = 3, [12] = 4, [13] = 5, [14] = 6, [15] = 7, },
	[9] = { [0] = 9, [1] = 8, [2] = 11, [3] = 10, [4] = 13, [5] = 12, [6] = 15, [7] = 14, [8] = 1, [9] = 0, [10] = 3, [11] = 2, [12] = 5, [13] = 4, [14] = 7, [15] = 6, },
	[10] = { [0] = 10, [1] = 11, [2] = 8, [3] = 9, [4] = 14, [5] = 15, [6] = 12, [7] = 13, [8] = 2, [9] = 3, [10] = 0, [11] = 1, [12] = 6, [13] = 7, [14] = 4, [15] = 5, },
	[11] = { [0] = 11, [1] = 10, [2] = 9, [3] = 8, [4] = 15, [5] = 14, [6] = 13, [7] = 12, [8] = 3, [9] = 2, [10] = 1, [11] = 0, [12] = 7, [13] = 6, [14] = 5, [15] = 4, },
	[12] = { [0] = 12, [1] = 13, [2] = 14, [3] = 15, [4] = 8, [5] = 9, [6] = 10, [7] = 11, [8] = 4, [9] = 5, [10] = 6, [11] = 7, [12] = 0, [13] = 1, [14] = 2, [15] = 3, },
	[13] = { [0] = 13, [1] = 12, [2] = 15, [3] = 14, [4] = 9, [5] = 8, [6] = 11, [7] = 10, [8] = 5, [9] = 4, [10] = 7, [11] = 6, [12] = 1, [13] = 0, [14] = 3, [15] = 2, },
	[14] = { [0] = 14, [1] = 15, [2] = 12, [3] = 13, [4] = 10, [5] = 11, [6] = 8, [7] = 9, [8] = 6, [9] = 7, [10] = 4, [11] = 5, [12] = 2, [13] = 3, [14] = 0, [15] = 1, },
	[15] = { [0] = 15, [1] = 14, [2] = 13, [3] = 12, [4] = 11, [5] = 10, [6] = 9, [7] = 8, [8] = 7, [9] = 6, [10] = 5, [11] = 4, [12] = 3, [13] = 2, [14] = 1, [15] = 0, },
}


function unu8 (str)
	return string.byte(string.sub(str, 1, 1))
end

function unu16 (str)
	local i = 0
	i = i + string.byte(string.sub(str, 1, 1))
	i = i + string.byte(string.sub(str, 2, 2)) * 256
	return i
end

function unu32 (str)
	local i = 0
	i = i + string.byte(string.sub(str, 1, 1))
	i = i + string.byte(string.sub(str, 2, 2)) * 256
	i = i + string.byte(string.sub(str, 3, 3)) * 65536
	i = i + string.byte(string.sub(str, 4, 4)) * 16777216
	return i
end

function u8 (i)
	return string.char(i)
end

function u16 (i)
	local f1 = math.floor(i / 256)
	local f2 = i % 256
	return string.char(f2) .. string.char(f1)
end

function u32 (i)
	local f1 = math.floor(i / 16777216)
	local r1 = i % 16777216
	local f2 = math.floor(r1 / 65536)
	local r2 = r1 % 65536
	local f3 = math.floor(r2 / 256)
	local f4 = r2 % 256
	return string.char(f4) .. string.char(f3) .. string.char(f2) .. string.char(f1)
end

function xor8 (i1, i2)
	local i1p2 = i1 % 16
	local i2p2 = i2 % 16
	local i1p1 = shrt_4[i1 - i1p2]
	local i2p1 = shrt_4[i2 - i2p2]
	
	return xort[i1p1][i2p1] * 16 + xort[i1p2][i2p2]
end	

function xor16 (i1, i2)
	local i1p1 = math.floor (i1 / 256)
	local i1p2 = i1 % 256
	local i2p1 = math.floor (i2 / 256)
	local i2p2 = i2 % 256
	
	return xor8 (i1p1, i2p1) * 256 + xor8 (i1p2, i2p2)
end

function xor32 (i1, i2)
	local i1p1 = math.floor (i1 / 65536)
	local i1p2 = i1 % 65536
	local i2p1 = math.floor (i2 / 65536)
	local i2p2 = i2 % 65536
	
	return xor16 (i1p1, i2p1) * 65536 + xor16 (i1p2, i2p2)
end

function build_key_stream (key, len)
	local keylen = string.len (key)
	
	local keypad = 4 - keylen % 4
	keylen = keylen + keypad
	key = key .. string.rep ('0', keypad)
	
	local keyseed = 0
	for i = 1, keylen, 4 do
		keyseed = keyseed + unu32 (string.sub (key, i, i+3) )
		if (keyseed >= 2147483648) then
			keyseed = keyseed - 2147483648
		end
	end
	
	math.randomseed (keyseed)
	
	local lenpad = 4 - len % 4
	if (lenpad == 4) then
		lenpad = 0
	end
	len = len + lenpad
	
	local key_index = 1
	local stream = ""
	
	for i = 1, len, 2 do
		local next_tok = math.random (0, 65535)
		local key_tok = unu16 (string.sub (key, key_index, key_index+1) )
		
		key_index = key_index + 2
		if (key_index >= keylen) then
			key_index = 1
		end
		
		stream = stream .. u16 (xor16 (next_tok, key_tok) )
	end
	
	math.randomseed (os.time() )
	
	return stream
end

function encrypt (key, data)
	local datalen = string.len (data)
	local key_stream = build_key_stream (key, datalen)
	
	local datapad = 4 - datalen % 4
	if (datapad < 4) then
		datalen = datalen + datapad
		data = data .. string.rep (string.char (0), datapad)
	end
	
	local stream = ""
	for i = 1, datalen do
		local b1 = string.byte (string.sub (data, i, i) )
		local b2 = string.byte (string.sub (key_stream, i, i) )
		stream = stream .. u8 (xor8 (b1, b2) )
	end
	
	return stream
end

function decrypt (key, data)
	return encrypt (key, data)
end