local util = require "./lua/util"

local string = string
local table = table

local setmetatable = setmetatable
local pairs = pairs
local ipairs = ipairs

local MF_PushString = MF_PushString

module (...)

-- ----------------------------------------------------------------------------

function StringTableClass ()
	return StringTable:new ()
end

StringTable = {
	filep = { mime = "STRI", ver = 0, hsz = 14 },
	strings = {},
}

function StringTable:new ()
	obj = {}
	setmetatable (obj, self)
	self.__index = self
	return obj
end

function StringTable:add (id, str)
	self.strings[id] = str
end

function StringTable:remove (id)
	self.strings[id] = nil
end

function StringTable:clear ()
	self.strings = {}
end

function StringTable:getString (id)
	if (self.strings[id] == nil) then
		return ""
	end
	
	return self.strings[id]
end

function StringTable:pushAll ()
	local sorted = {}
	for k, v in pairs (self.strings) do
		table.insert (sorted, { key = k, val = v})
	end
	sorted = util.paramsort (sorted, 'key')
	
	for k, v in ipairs (sorted) do
		MF_PushString (v.key, v.val)
	end
end

function StringTable:import (data, ver)
	local offStr = util.unu32(string.sub(data, 1, 4)) - 8
	local numStr = util.unu16(string.sub(data, 5, 6))
	
	local data_idx = 6
	for i = 1, numStr do
		local id = util.unu16(string.sub(data, data_idx+1, data_idx+2))
		local len = util.unu32(string.sub(data, data_idx+3, data_idx+6))
		local str = util.unfixstr(string.sub(data, data_idx+7, data_idx+6+len))
		
		self.strings[id] = str
		
		data_idx = data_idx + 6 + len
	end
end

function StringTable:export ()
	local data = ""
	local dataStr = ""
	local numStr = 0
	
	for k, v in pairs (self.strings) do
		dataStr = dataStr .. util.u16(k) .. util.u32(string.len(v)) .. v
		numStr = numStr + 1
	end
	
	data = self.filep.mime .. util.u16(self.filep.ver) .. util.u16(self.filep.hsz) ..
		   util.u32(self.filep.hsz) .. util.u16(numStr) .. dataStr
	return data
end