local util = require "./lua/util"

local math = math
local table = table
local string = string

local setmetatable = setmetatable
local ipairs = ipairs
local DoCall = DoCall

module (...)

-- ----------------------------------------------------------------------------

function ZoneClass ()
	return Zone:new()
end

Zone = {
	filep = { mime = "ZONE", ver = 0, szBlock = 18, szZone = 40 },
	zones = {},
	lastIndex = 0,
	ttc = nil,
}

function Zone:new ()
	obj = {}
	setmetatable (obj, self)
	self.__index = self
	return obj
end

function Zone:add (x1, y1, x2, y2)
	local newzone = {
		name = 'Zone ' .. util.tostring( self.lastIndex ),
		x = math.min(x1,x2),
		y = math.min(y1,y2),
		width = math.max(x1,x2) - math.min(x1,x2),
		height = math.max(y1,y2) - math.min(y1,y2),
	}
	
	--setmetatable( newzone, setmt )

	table.insert( self.zones, newzone )
		
	self.lastIndex = self.lastIndex + 1
	local zonen = #self.zones
	self:_draw( zonen, newzone.name, newzone.x, newzone.y, newzone.width, newzone.height )
end	

function Zone:clear (id)
	table.remove( self.zones, id )
end
	
function Zone:update (name, newname, x, y, w, h)
	for i = 1,#self.zones do
		v = self.zones[i]
		if (v.name == name) then
			v.name = newname
			v.x = util.tonumber(x)
			v.y = util.tonumber(y)
			v.width = util.tonumber(w)
			v.height = util.tonumber(h)
			self:_draw( i, newname, x, y, w, h )
		end
	end
end			

function Zone:_draw (id, name, x, y, w, h)
	if (self.ttc ~= nil) then
		x, y = self.ttc (x, y)
	end
	DoCall( "ZoneDrawBox", id, name, x, y, w, h )
end
	
function Zone:drawIndex ()
	DoCall( "ZoneResetIndex" )
	for i = 1,#self.zones do
		local lzone = self.zones[i]
		DoCall( "ZoneDrawIndex", i, lzone.name, lzone.x, lzone.y, lzone.width, lzone.height )
	end
	DoCall( "ZoneEndIndex", #self.zones )
end
	
function Zone:drawBoxes ()
	DoCall( "ZoneResetBoxes" )
	for i = 1,#self.zones do
		local lzone = self.zones[i]
		self:_draw( i, lzone.name, lzone.x, lzone.y, lzone.width, lzone.height )
	end
	DoCall( "ZoneEndBoxes" )
end
	
function Zone:registerZone (zn)
	zn = util.tonumber(zn)
	if ( zn < 1 ) then
		zn = 1
	elseif ( zn > #self.zones ) then
		zn = #self.zones
	end
	
	DoCall( "ActiveZone", zn, self.zones[zn].name )
	return zn
end
	
function Zone:import (data, ver)
	local offZone = util.unu32(string.sub(data, 1, 4)) - 8
	local numZone = util.unu32(string.sub(data, 5, 8))
	local szZone = util.unu16(string.sub(data, 9, 10))
	self.zones = {}
	self.lastIndex = 0
		
	for iZone = 1, numZone do
		local addrZone = offZone + (iZone - 1) * szZone
		local name = util.unfixstr(string.sub(data, addrZone+1, addrZone+32))
		local x = util.unu16(string.sub(data, addrZone+33, addrZone+34))
		local y = util.unu16(string.sub(data, addrZone+35, addrZone+36))
		local width = util.unu16(string.sub(data, addrZone+37, addrZone+38))
		local height = util.unu16(string.sub(data, addrZone+39, addrZone+40))
		self:add( x, y, x+width, y+height )
		self.zones[#self.zones].name = name
	end
	
	self:registerZone( 1 )
end
	
function Zone:export ()
	local data = ""
	local dataZone = ""
	local idxZone = 0
	
	for k,v in ipairs( self.zones ) do
		dataZone = dataZone .. util.fixstr(v.name, 32) .. util.u16(v.x) .. util.u16(v.y) .. util.u16(v.width) .. 
		           util.u16(v.height)
		idxZone = idxZone + 1
	end
	
	data = self.filep.mime .. util.u16(self.filep.ver) .. util.u16(self.filep.szBlock) .. 
		   util.u32(self.filep.szBlock) .. util.u32(idxZone) .. util.u16(self.filep.szZone)
	return data .. dataZone
end
	
function Zone:reset ()
	self.zones = {}
	self.lastIndex = 0
end